// SPDX-FileCopyrightText: 2019-2022 Badwolf Authors <https://hacktivis.me/projects/badwolf>
// SPDX-License-Identifier: BSD-3-Clause

#include <stdint.h> // uint64_t

#define BADWOLF_CTX_SIZ 7
void fmt_context_id(uint64_t num, char *out);
